/*
    (C) SergeySoft 2023 z-print@bk.ru
    Re-spread book from Scan - делит отсканированные шитые на скобу СПУСКИ А3, в книгу а4 с правильным порядком страниц
    Re-face A3 to A4 - делит РАЗВОРОТЫ на отдельные страницы
*/    

if (app.viewerVersion < 10) {
    
    app.addMenuItem({ cName: "Re-face A3 to A4", cUser: "Re-face A3 to A4", cParent: "Tools", cExec:"splitFaces()", cEnable: "event.rc = (event.target != null);"});
    app.addMenuItem({ cName: "Re-spread book from Scan", cUser: "Re-spread book from Scan", cParent: "Tools", cExec:"splitAndReoder()", cEnable: "event.rc = (event.target != null);"});
    
} else {

    app.addToolButton({ cName: "Re-face A3 to A4", cLabel: "Re-face A3 to A4", cExec: "splitFaces()", cEnable: "event.rc = (event.target != null);"});   
    app.addToolButton({ cName: "Re-spread book from Scan", cLabel: "Re-spread book from Scan", cExec: "splitAndReoder()", cEnable: "event.rc = (event.target != null);"});   
}

const SPREADSIZE = 4; //  количество страниц в комплекте А3


//  режет РАЗВОРОТЫ на левые и правые страницы, 
splitFaces = app.trustedFunction(function() {
    splitInHalf("_A4.pdf");
});


//  режет СПУСКИ тетрадей любого объема пополам и переупорядочивает страницы в верном порядке
splitAndReoder = app.trustedFunction(function() {
    //  спуск состоит из лица и оборота (n раз), если нечетное количество листов, то отсканировано с ошибкой
    //  вываливаемся с ошибкой 
    if (this.numPages % 2) {
        app.alert ("Error on Scan\nIt must contain EVEN page count");
        return;
    }
    //  порежем копию исходного файла пополам и сделаем ее активной
    splitInHalf("_reodered.pdf");

    try {

        var spreadCount = this.numPages / SPREADSIZE; //  количество двусторонних А3

        var lastPage = this.numPages - 1; //  номер последней страницы

        var t = app.thermometer

        t.duration = spreadCount;
        t.begin();

        for (i = 0; i < spreadCount; i++) {

            t.value =  i;
            t.text = "Reordering spread: " + i.toString();
            if (t.cancelled) break;
            
            //  переставляем первую страницу спуска как последнюю книги
            //  вставляем в конец файла ( lastPage ), 
            // c учетом номера спуска (i * 2 - т.к. в начале все время исчезают по 2 страницы в спуске)
            // и отступом 0 для последней страницы, 1 - для предпоследней
            this.movePage(i * 2 + 0, lastPage - i * 2 - 0);
            //  переставляем последнюю (3) страницу спуска как предпоследнюю книги. 
            // 2 - т.к. нумерация изменилась прошлой итерации
            this.movePage(i * 2 + 2, lastPage - i * 2 - 1);

        }
        t.end();

        // app.endPriv();
    }
    
    catch(e) {
        app.alert(e);
    };
    
})


//  Задваивает количество страниц и каждую пополам, оставляя правую или левую стороны
function splitInHalf(fileNameSuffix) {
    app.beginPriv();
    
    try {
        //  сохраним под новым именем
        var newName = this.path;
        var filename = newName.replace(/\.pdf/i, fileNameSuffix); // сохраним копию документа и будем работать с копией

        this.saveAs(filename);

        // сдублируем страницы, чтобы потом сделать из них левые/правые страницы спуска
        var num = this.numPages; //  текущее количество страниц в файле
        var t = app.thermometer
        t.duration = num;
        t.begin();

        for (var i = 0; i < num; i++) {
            
            t.value =  i;
            t.text = "Duplicate page: " + i.toString();
            
            if (t.cancelled) break;
            
            k = i * 2;
            this.insertPages({ nPage: k, cPath:newName, nStart:i });
        };

        t.end();


        //  сделаем левые/правые страницы
        //  можно воткнуть порезку в предыдущий цикл, где вставляются страницы, но преимущество по скорости минимально, а читаться будет хуже
        var num2 = this.numPages; // текущее количество страниц в файле. Чтобы на каждом проходе не обращаться к this.numPages 
        
        t.duration = num2;
        t.begin();

        for (i = 0; i < num2; i += 2) {
                
            t.value =  i;
            t.text = "Define Left/Right Pages: " + i.toString();
            if (t.cancelled) break;

            var aRect = this.getPageBox("Media", i);
            var halfW = (aRect[2] - aRect[0]) / 2;


            //  левые страницы
            var oRect = aRect.concat();
            oRect[2] = oRect[0] + halfW;
            this.setPageBoxes("Media", i, i, oRect);
            this.setPageBoxes("Trim",  i, i, oRect);

            //  правые страницы
            oRect = aRect.concat();
            oRect[0] = oRect[2] - halfW;
            this.setPageBoxes("Media", i + 1, i + 1, oRect);
            this.setPageBoxes("Trim",  i + 1, i + 1, oRect);
        };

        t.end();
    
        app.endPriv();
    }
    
    catch(e) {
        app.alert(e);
    };
    
}

